--[[
	Helper Functions
	v1.9
]]

-- Let's us sort tables!
function MiniMount_pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end

-- Get the current Riding Skill Level
function MiniMount_GetRidingSkill()
	-- Gather Base Names. Used for Detecting spells...
	local Apprentice_Name = GetSpellInfo(33388);		-- Regular
	local Journeyman_Name = GetSpellInfo(33391);		-- Epic
	local Expert_Name = GetSpellInfo(34090);		-- Regular Flying
	local Artisan_Name = GetSpellInfo(34091);		-- Epic Flying
	local Master_Name = GetSpellInfo(90265);		-- Fast Flying

	-- Check to see if the spells are in our spellbook.
	local Test_Apprentice = GetSpellInfo(Apprentice_Name);
	local Test_Journeyman = GetSpellInfo(Journeyman_Name);
	local Test_Expert = GetSpellInfo(Expert_Name);
	local Test_Artisan = GetSpellInfo(Artisan_Name);
	local Test_Master = GetSpellInfo(Master_Name);		-- Don't actually need to test for Master, since it uses the same mounts as Artisan.

	local skill = 1;
	if (Test_Master ~= nil) then
		skill = 375;
	elseif (Test_Artisan ~= nil) then
		skill = 300;
	elseif (Test_Expert ~= nil) then
		skill = 225;
	elseif (Test_Journeyman ~= nil) then
		skill = 150;
	elseif (Test_Apprentice ~= nil) then
		skill = 75;
	end

	return skill;
end

function MiniMount_CanFly()
	SetMapToCurrentZone();
	local content = GetCurrentMapContinent();

	local FML_Name = GetSpellInfo(90267);	-- Flight Master's License (Azeroth Flying)
	local CWF_Name = GetSpellInfo(54197);	-- Cold Weather Flying (Northrend Flying)

	local Test_FML = GetSpellInfo(FML_Name);
	local Test_CWF = GetSpellInfo(CWF_Name);

	if (Test_FML ~= nil) and ((content == 1) or (content == 2) or (content == 5)) then
		return true;
	elseif (MiniMount_GetRidingSkill() >= 225) and (content == 3) then
		return true;
	elseif (Test_CWF ~= nil) and (content == 4) then
		return true;
	end
	return nil;
end

-- Do Preperations for Classes (If possible)
function MiniMount_MountPrepare()
	local _, Class = UnitClass("player");

	-- Remove a Druid's Shapeshift
	if Class == "DRUID" then
		return "\n/cancelform [btn:1]";
	-- Remove a Shaman's Shapeshift
	elseif Class == "SHAMAN" then
		return "\n/cancelform [btn:1]";
	end

	return "";
end

-- Is it possible to fly at all in this zone?
function MiniMount_FlyZone(cc,cnf)
	local output = nil;
	local mode = MiniMount_ZoneSmart();
	local zone = GetZoneText();

	if (cc == 3) or ((cc == 4) and (cnf == true)) then
		local d = false;
		if (cc == 4) and (zone == MINIMOUNT_ZONE_WINTERGRASP) and (GetWintergraspWaitTime() == nil) then
			return nil;
		else 
			output = true;
			if mode == "ground" then
				output = nil;
			end
		end
	end

	return output;
end

-- Set Holiday Info
function MiniMount_SetHoliday()
	MiniMount_Msg("Setting Holiday...", "debug", 1);
	local _, _, day, _ = CalendarGetDate();
	local nume = CalendarGetNumDayEvents(0,day);
	for x=1,nume do
		local Name = CalendarGetHolidayInfo(0, day, x);
		MiniMount_Msg("Detected Holiday: \""..tostring(Name).."\"", "debug", 1);
		if (Name == MINIMOUNT_HOLIDAYS_LOVEISINTHEAIR) then
			MiniMount_Holiday = MINIMOUNT_HOLIDAYS_LOVEISINTHEAIR;
		elseif (Name == MINIMOUNT_HOLIDAYS_BREWFEST) then
			MiniMount_Holiday = MINIMOUNT_HOLIDAYS_BREWFEST;
		elseif (Name == MINIMOUNT_HOLIDAYS_HALLOWSEND) then
			MiniMount_Holiday = MINIMOUNT_HOLIDAYS_HALLOWSEND;
		elseif (Name == MINIMOUNT_HOLIDAYS_WINTERVEIL) then
			MiniMount_Holiday = MINIMOUNT_HOLIDAYS_WINTERVEIL;
		end
	end
	MiniMount_Msg("Final Holiday Setting: "..tostring(MiniMount_Holiday), "debug", 1);
end

-- Timer function
function MiniMount_Timer(name,targettime,functiontodo, ...)
	local framename = "MiniMountFrame"..name;
	if getglobal(framename) and (getglobal(framename):GetScript("OnUpdate") ~= nil) then
		return;
	end

	local funcvar1, funcvar2, funcvar3, funcvar4 = ...;

	local frame = CreateFrame("Frame", framename);
	
	local totalElapsed = 0.0;
	local tickcount = 0;

	local function MiniMount_onUpdate(self, elapsed)
		totalElapsed = totalElapsed + elapsed;
		if (totalElapsed < 1) then return; end
		totalElapsed = totalElapsed - floor(totalElapsed);
		tickcount = tickcount + 1;

		if (tickcount == targettime) then
			local result;
			if (funcvar4 ~= nil) then
				result = functiontodo(funcvar1, funcvar2, funcvar3, funcvar4);
			elseif (funcvar3 ~= nil) then
				result = functiontodo(funcvar1, funcvar2, funcvar3);
			elseif (funcvar2 ~= nil) then
				result = functiontodo(funcvar1, funcvar2);
			elseif (funcvar1 ~= nil) then
				result = functiontodo(funcvar1);
			else
				result = functiontodo();
			end
			frame:SetScript("OnUpdate", nil);
			return result;
		end
		-- Enable for timing debugging
		-- ChatFrame1:AddMessage("tick... "..tickcount);
	end
	frame:SetScript("OnUpdate", MiniMount_onUpdate);
	frame:Show();
end

-- ItemID info
function MiniMount_GetItemID(itemlink)
	if (itemlink == nil) or (strfind(itemlink,":") == nil) then
		return;
	end

	local justItemId = string.gsub(itemlink,".-\124H([^\124]*)\124h.*", "%1");
	local type, itemid, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":",justItemId);

	return itemid;
end

function MiniMount_AddMountToDB(db,MountNum)
	local indb = false;

	for x=1,#db do
		if (db[x] == MountNum) then
			indb = true;
			break;
		end
	end

	if indb == false then
		tinsert(db, (#db+1), MountNum);
	end
end